package com.gct.dataengine.collector.config;

import com.gct.dataengine.collector.properties.BindTaskTriggerConfigurationProperties;
import com.gct.dataengine.collector.properties.HeartBeatTriggerConfigurationProperties;
import com.gct.dataengine.collector.quartztask.BindTaskJob;
import com.gct.dataengine.collector.quartztask.HeartBeatJob;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import static org.quartz.CronScheduleBuilder.cronSchedule;

/**
 * @author xusirui 2022/4/13
 */
@Configuration
@EnableConfigurationProperties({HeartBeatTriggerConfigurationProperties.class, BindTaskTriggerConfigurationProperties.class})
@Slf4j
public class QuartzTaskConfig {

    private final HeartBeatTriggerConfigurationProperties heartBeatProperties;

    private final BindTaskTriggerConfigurationProperties bindTaskProperties;


    public QuartzTaskConfig(HeartBeatTriggerConfigurationProperties heartBeatProperties,BindTaskTriggerConfigurationProperties bindTaskProperties) {
        this.heartBeatProperties = heartBeatProperties;
        this.bindTaskProperties = bindTaskProperties;
    }

    @Bean
    public JobDetail bindTaskJobDetail(){

        try {
            return JobBuilder.newJob(BindTaskJob.class)
                    .withIdentity(bindTaskProperties.getJobName(), bindTaskProperties.getJobGroup())
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("Create JobDetail bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),bindTaskProperties.getJobName(),bindTaskProperties.getJobGroup());
        }
        return null;
    }

    @Bean
    public Trigger bindTaskJobTrigger() {
        try {
            return TriggerBuilder.newTrigger()
                    .forJob(bindTaskProperties.getJobName(), bindTaskProperties.getJobGroup())
                    .withSchedule(cronSchedule(bindTaskProperties.getCronStr())).build();
        } catch (Exception e) {
            log.error("Create Trigger bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),bindTaskProperties.getJobName(),bindTaskProperties.getJobGroup());
            throw e;
        }
    }

    @Bean
    public JobDetail heartBeatJobDetail(){

        try {
            return JobBuilder.newJob(HeartBeatJob.class)
                    .withIdentity(heartBeatProperties.getJobName(), heartBeatProperties.getJobGroup())
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("Create JobDetail bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),heartBeatProperties.getJobName(),heartBeatProperties.getJobGroup());
        }
        return null;
    }

    @Bean
    public Trigger heartBeatJobTrigger() {
        try {
            return TriggerBuilder.newTrigger()
                    .forJob(heartBeatProperties.getJobName(), heartBeatProperties.getJobGroup())
                    .withSchedule(cronSchedule(heartBeatProperties.getCronStr())).build();
        } catch (Exception e) {
            log.error("Create Trigger bean failed:{},jobName->{},jobGroup->{}", e.getMessage(),heartBeatProperties.getJobName(),heartBeatProperties.getJobGroup());
            throw e;
        }
    }
}
