package com.gct.dataengine.collector.quartztask;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gct.dataengine.collector.entity.vo.CollectorTaskInfoVo;
import com.gct.dataengine.collector.entity.vo.MachineServerState;
import com.gct.dataengine.collector.feign.CentralClient;
import com.gct.dataengine.collector.feign.RegisterClient;
import com.gct.dataengine.collector.properties.CollectorProperties;
import com.gct.dataengine.collector.service.file.DriverFileService;
import com.gct.dataengine.collector.service.task.TemplateConvert;
import com.gct.dataengine.collector.service.file.TemplateFileService;
import com.gct.dataengine.collector.service.task.thirdlevel.TemplateTask;
import com.gct.dataengine.collector.service.task.TemplateTasksManage;
import com.gct.dataengine.template.entity.xml.LotServer;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.net.Inet4Address;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author xusirui 2022/4/21
 */
@Slf4j
public class BindTaskJob extends QuartzJobBean {

    @Autowired
    RegisterClient registerClient;

    @Autowired
    CollectorProperties collectorProperties;

    @Autowired
    TemplateTasksManage templateTasksManage;

    @Autowired
    TemplateFileService templateFileService;

    @Autowired
    DriverFileService driverFileService;

    @Autowired
    TemplateConvert templateConvert;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        int times = 10;
        while (times-->0){
            CollectorTaskInfoVo collectorTaskInfoVo;
            try{
                collectorTaskInfoVo = registerClient.pullCollectorTaskInfo();
            }catch (RuntimeException e){
                log.error("pullCollectorTaskInfo failed, may be register is outline, cause by {}",e.getMessage());
                break;
            }
//            System.out.println(collectorTaskInfoVo);
            templateTasksManage.syncRemoteTaskList(collectorTaskInfoVo.getTaskCollectorMap(),collectorProperties.getId());
            String tryBindTaskId = balanceAlgo(collectorTaskInfoVo);
            if(Objects.isNull(tryBindTaskId))break;
            Boolean isBindSuccess = registerClient.bind(collectorProperties.getId(), tryBindTaskId, collectorTaskInfoVo.getUuidVersion());
            if (isBindSuccess){
                //download template file and driver file,parse template to task,add circle task to schedule, throw RuntimeException while failed
                String templateFilePath = templateFileService.loadTemplateFile(tryBindTaskId);
                driverFileService.loadDriverFile(templateFilePath);
                TemplateTask templateTask = templateConvert.convert(tryBindTaskId,templateFilePath);
                templateTasksManage.addTask(tryBindTaskId,templateTask);
                break;
            }
        }
        if (times==0){
            log.info("register server busy, please wait next bind task");
        }
    }

    public String balanceAlgo(CollectorTaskInfoVo collectorTaskInfoVo) {
        for (Map.Entry<Object, Object> entry : collectorTaskInfoVo.getTaskCollectorMap().entrySet()) {
            if (entry.getValue().equals("null")) return entry.getKey().toString();
        }
        return null;
    }



}
