package com.gct.dataengine.collector.service.file;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.oss.OSSClient;
import com.gct.dataengine.collector.entity.vo.DriverLocation;
import com.gct.dataengine.collector.entity.vo.FileLocationVo;
import com.gct.dataengine.collector.feign.CentralClient;
import com.gct.dataengine.collector.feign.SuitcaseClient;
import com.gct.dataengine.collector.service.task.TemplateConvert;
import com.gct.dataengine.template.entity.xml.TemplateDefinition;
import com.gct.dataengine.template.util.DeviceXmlConfigConvert;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Arrays;
import java.util.Objects;

/**
 * @author xusirui 2022/7/12
 */
@Service
@Slf4j
public class DriverFileService {

    @Autowired
    SuitcaseClient suitcaseClient;

    @Autowired
    CentralClient centralClient;

    @Autowired
    OSSClient ossClient;

    private static final String baseDriverFilePath = System.getProperty("user.dir") + "/.temp/driver/";

    public DriverFileService(){
        super();
        File driverFileDir = new File(baseDriverFilePath);
        if (!driverFileDir.exists() && !driverFileDir.mkdirs()) throw new RuntimeException("driver file base dir create failed");
    }


    public Boolean loadDriverFile(String templateFilePath){
        DeviceXmlConfigConvert template = new DeviceXmlConfigConvert(templateFilePath);
        TemplateDefinition templateDefinition = template.read();
        Boolean isAllDownload= Arrays.stream(templateDefinition.getLotServers()).allMatch(lotServer -> {
            String res = centralClient.searchClassConfig(lotServer.getDriverClassName(), lotServer.getDriverNamespace());
            Object driverLocationData = JSONObject.parseObject(res, Result.class).getData();
            if (Objects.isNull(driverLocationData)){
                log.error("can not find driver location in server => central");
                return false;
            }
            DriverLocation driverLocation = JSONObject.parseObject(driverLocationData.toString(), DriverLocation.class);
            res = suitcaseClient.getFileLocation(driverLocation.getDriverFileName(),driverLocation.getLimitVersion());
            Object driverFileUrlData = JSONObject.parseObject(res,Result.class).getData();
            if (Objects.isNull(driverFileUrlData)){
                log.error("can not find driver file url in server => suitcase");
                return false;
            }
            FileLocationVo fileLocation = JSONObject.parseObject(driverFileUrlData.toString(), FileLocationVo.class);
            File file = new File(baseDriverFilePath + fileLocation.getFileName());
            if (file.exists()){
                lotServer.setDriverPath(file.getPath());
                return true;
            }
            else {
                boolean isDownload = ossClient.getFileToPath(fileLocation.getBucketName(), fileLocation.getFileName(), file.getPath());
                if (isDownload) {
                    lotServer.setDriverPath(file.getPath());
                }else {
                    log.error("driver download from oss failed");
                    return false;
                }
            }
            return true;
        });
        log.info("load template {}",templateDefinition);
        template.write(templateDefinition);
        return isAllDownload;
    }
}
