package com.gct.dataengine.collector.service.file;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.common.oss.OSSClient;
import com.gct.dataengine.collector.entity.vo.TemplateFileLocationVo;
import com.gct.dataengine.collector.feign.SuitcaseClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Objects;

/**
 * @author xusirui 2022/4/25
 */
@Service
public class TemplateFileService {

    @Autowired
    SuitcaseClient suitcaseClient;

    @Autowired
    OSSClient ossClient;

    private static final String baseTempFilePath = System.getProperty("user.dir") + "/.temp/template/";

    public TemplateFileService(){
        super();
        File tempFileDir = new File(baseTempFilePath);
        if (!tempFileDir.exists() && !tempFileDir.mkdirs()) throw new RuntimeException("template file base dir create failed");
    }

    /**
     * get task detail message by data-center feign interface
     * @param taskId
     */
    public String loadTemplateFile(String taskId){
        // todo :  query data-center feign interface get task oss bucketName and objectName
        String res = suitcaseClient.getOssTemplateFileById(taskId);
        TemplateFileLocationVo location = JSONObject.parseObject(JSONObject.parseObject(res, Result.class).getData().toString(), TemplateFileLocationVo.class);
        if (Objects.isNull(location.getFileName())||Objects.isNull(location.getBucketName())){
            throw new RuntimeException("template file location not found in oss service");
        }
        return loadTemplateFileFromOss(location.getBucketName(),location.getFileName());
    }

    private String loadTemplateFileFromOss(String bucketName,String objectName){
        String localPath = baseTempFilePath +objectName;
        boolean isDownload = ossClient.getFileToPath(bucketName, objectName, localPath);
        if (!isDownload){
            throw new RuntimeException("template file download failed");
        }
        // todo : save file to a general directory
        return localPath;
    }
}
