package com.gct.dataengine.collector.service.task;

import com.gct.dataengine.template.entity.enums.Constant;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static org.quartz.CronScheduleBuilder.cronSchedule;

/**
 * @author xusirui 2022/5/11
 */
@Service
@Slf4j
public class QuartzJobPool {

    @Autowired
    Scheduler scheduler;

    public boolean addJob(String jobName, String jobGroup, String triggerName, String triggerGroup, Object jobParam, Integer executeInterval,Class<? extends Job> quartzJob) {
        try {
            JobDetail jobDetail = JobBuilder
                    .newJob(quartzJob)
                    .withIdentity(jobName, jobGroup)
                    .build();
            jobDetail.getJobDataMap().put(Constant.JOB_PARAM_KEY, jobParam);
            Trigger trigger = TriggerBuilder
                    .newTrigger()
                    .withIdentity(triggerName, triggerGroup)
                    .startNow()
                    .withSchedule(SimpleScheduleBuilder.simpleSchedule().withIntervalInSeconds(executeInterval).repeatForever())
                    .build();
            scheduler.scheduleJob(jobDetail, trigger);
        } catch (SchedulerException e) {
            log.error("quartz job add failed, cause by {}", e.getMessage());
            return false;
        }
        return true;
    }

    public boolean removeJob(String jobName, String jobGroup, String triggerName, String triggerGroup) {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(triggerName, triggerGroup);
            scheduler.pauseTrigger(triggerKey);
            scheduler.unscheduleJob(triggerKey);
            scheduler.deleteJob(JobKey.jobKey(jobName, jobGroup));
        } catch (SchedulerException e) {
            log.error("quartz job remove failed, cause by {}", e.getMessage());
            return false;
        }
        return true;
    }

}
