package com.gct.dataengine.collector.service.task;

import com.gct.dataengine.collector.service.task.secondlevel.CircleTask;
import com.gct.dataengine.collector.service.task.thirdlevel.TemplateTask;
import com.gct.dataengine.template.entity.enums.Constant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author xusirui 2022/4/24
 */
@Service
@Slf4j
public class TemplateTasksManage {

    @Autowired
    QuartzJobPool quartzJobPool;

    private ConcurrentHashMap<String, TemplateTask> taskPool = new ConcurrentHashMap<>();

    public void addTask(String taskId, TemplateTask templateTask){
        for (CircleTask circleTask : templateTask.getCircleTasks()) {
            quartzJobPool.addJob(
                    circleTask.getTaskId(),
                    Constant.DATA_BLOCK_JOB_GROUP_NAME,
                    circleTask.getTaskId(),
                    Constant.DATA_BLOCK_TRIGGER_GROUP_NAME,
                    circleTask,
//                    String.format("%d/%d * * * * ? ", startPoint,circleTask.getExecuteInterval()),
                    circleTask.getExecuteInterval(),
                    circleTask.getQuartzJobClass()
            );
        }
        taskPool.put(taskId, templateTask);
    }

    /**
     * remove task from task pool
     * @param templateTaskId
     */
    public void removeTask(String templateTaskId){
        TemplateTask templateTask = taskPool.get(templateTaskId);
        if (Objects.nonNull(templateTask)){
            for (CircleTask circleTask : templateTask.getCircleTasks()) {
                quartzJobPool.removeJob(
                        circleTask.getTaskId(),
                        Constant.DATA_BLOCK_JOB_GROUP_NAME,
                        circleTask.getTaskId(),
                        Constant.DATA_BLOCK_TRIGGER_GROUP_NAME
                );
            }
            taskPool.remove(templateTaskId);
        }
    }

    /**
     * while remote task has update, this collector will unbind the update task
     * @param taskCollectorMap   remote taskCollectorMap
     * @param collectorId   this collector Id
     */
    public void syncRemoteTaskList(Map<Object,Object> taskCollectorMap,String collectorId){
        Set<String> bindTaskIdSet = taskCollectorMap.entrySet()
                .stream()
                .filter(entry -> entry.getValue().equals(collectorId))
                .map(entry -> entry.getKey().toString())
                .collect(Collectors.toSet());
        List<String> expireTaskIds = taskPool.keySet().stream().filter(x -> !bindTaskIdSet.contains(x)).collect(Collectors.toList());
        expireTaskIds.forEach(this::removeTask);
        if(expireTaskIds.size()>0)log.info("remove expire task {}",expireTaskIds);
    }
}
