package com.gct.dataengine.collector.service.task.secondlevel;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.dataengine.collector.entity.BlockData;
import com.gct.dataengine.collector.feign.CauldronClient;
import com.gct.dataengine.collector.service.task.CollectedDataProcessor;
import com.gct.dataengine.collector.service.task.firstlevel.CollectTask;
import com.gct.dataengine.collector.service.task.firstlevel.ValueCollectTask;
import com.gct.driver.lotpi.DataBit;
import com.gct.driver.lotpi.DataBlock;
import com.gct.driver.lotpi.extent.Connector;
import lombok.extern.slf4j.Slf4j;
import org.quartz.Job;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xusirui 2022/3/31
 */
@Slf4j
public class DataBlockTask extends CircleTask {

    public DataBlockTask(Integer executeInterval, Class<? extends Job> quartzJobClass, CollectedDataProcessor collectedDataProcessor) {
        super(executeInterval, quartzJobClass, collectedDataProcessor);
    }

    @Override
    public void runTask() {
        List<BlockData.DataTag> tags = new ArrayList<>();
        List<BlockData.DataValue> values = new ArrayList<>();
        long st = System.currentTimeMillis();
        List<ValueCollectTask> valueCollectTasks = new ArrayList<>();
        taskList.forEach(collectTask->{
            if (collectTask instanceof ValueCollectTask){
                valueCollectTasks.add((ValueCollectTask) collectTask);
            }
        });
        List<DataBit[]> dataBitsList = valueCollectTasks.stream().map(ValueCollectTask::getDataBits).collect(Collectors.toList());
        List<Connector> connectorList = valueCollectTasks.stream().map(ValueCollectTask::getConnector).collect(Collectors.toList());
        Map<Connector,List<DataBit>> connBitsMap = new HashMap<>();
        for (int i = 0; i < dataBitsList.size(); i++) {
            DataBit[] dataBits = dataBitsList.get(i);
            Connector connector = connectorList.get(i);
            connBitsMap.putIfAbsent(connector,new ArrayList<>());
            for (int j = 0; j < dataBits.length; j++) {
                connBitsMap.get(connector).add(dataBits[j]);
            }
        }
        Map<Connector,DataBlock> connBlockMap= new HashMap<>();
        connBitsMap.forEach((connector,dataBits)->{
            log.info("dataBits {}",JSONObject.toJSONString(dataBits.toArray(new DataBit[]{})));
            DataBlock dataBlock = connector.read(dataBits.toArray(new DataBit[]{}));
            connBlockMap.put(connector,dataBlock);
        });
        valueCollectTasks.forEach(valueCollectTask -> {
            valueCollectTask.setDataBlock(connBlockMap.get(valueCollectTask.getConnector()));
        });
        log.info("dataBlock {},databits {}",JSONObject.toJSONString(connBlockMap),JSONObject.toJSONString(dataBitsList));
        taskList.stream()
                //.parallel()  //not support  parallel query by a connnector
                .map(CollectTask::run)
                .forEach(o -> {
                    if (o instanceof BlockData.DataTag) tags.add((BlockData.DataTag) o);
                    else if (o instanceof BlockData.DataValue) values.add((BlockData.DataValue) o);
                });

        log.info("read data from RTU cost {} ms",System.currentTimeMillis() - st);
        st = System.currentTimeMillis();
        collectedDataProcessor.writeBlockData(new BlockData(values.toArray(new BlockData.DataValue[0]), tags.toArray(new BlockData.DataTag[0])));
        log.info("write data to influx DB cost {} ms",System.currentTimeMillis() - st);
    }
}
